#!/usr/bin/perl

use strict;
use warnings;
use LWP::Simple;
use File::Basename;
use Getopt::Long;
use Pod::Usage;

delete @ENV{ grep /^LC_/ => keys %ENV };
$ENV{LANG}   = "C";
$ENV{LC_ALL} = "C";

sub version($$);
sub get_temp($$);
sub report($$$);

my %ERRORS = (
    OK        => 0,
    WARNING   => 1,
    CRITICAL  => 2,
    UNKNOWN   => 3,
    DEPENDENT => 4
);

my $ME      = basename $0;
my $NAME    = "ROOMTEMP";
my $VERSION = "0.1";

my %opt = (
    warning  => 35.00,
    critical => 40.00,
    url      => 'http://thermo.local.site/status.xml',
    search   => '<temp>[+-]\s+([\d\.]+)</temp>'
);

MAIN: {
    Getopt::Long::Configure('bundling');
    GetOptions(
        "w|warning=f"  => \$opt{warning},
        "c|critical=f" => \$opt{critical},
        "u|url=s"      => \$opt{url},
        "s|search=s"   => \$opt{search},
        "h|help" => sub { pod2usage( -verbose => 1, -exitval => $ERRORS{OK} ) },
        "m|man" => sub { pod2usage( -verbose => 2, -exitval => $ERRORS{OK} ) },
        "V|version" => sub { version( $ME, $VERSION ); exit $ERRORS{OK}; }
    ) or pod2usage( -verbose => 1, -exitval => $ERRORS{CRITICAL} );

    my ($temp) = get_temp( \$opt{url}, \$opt{search} );
    report( \$temp, \$opt{warning}, \$opt{critical} );
}

sub get_temp($$) {
    my ( $url, $search ) = @_;
    my $temp;
    my $content = get($$url);
    if ( !$content ) {
        print "$NAME UNKNOWN: can't get status informations from URL\n";
        exit $ERRORS{UNKNOWN};
    }
    $content =~ /$$search/;

    if ( !$1 ) {
        print "$NAME UNKNOWN: can't get temperature from search string\n";
        exit $ERRORS{UNKNOWN};
    }
    $temp = sprintf( "%.2f", $1 );
    return $temp;
}

sub report($$$) {
    my ( $temp, $warning, $critical ) = @_;
    $$warning  = sprintf( "%.2f", $$warning );
    $$critical = sprintf( "%.2f", $$critical );

    if ( $$temp >= $$critical ) {
        print
          "$NAME CRITICAL: +$$temp C | roomtemp=$$temp;$$warning;$$critical\n";
        exit $ERRORS{CRITICAL};
    }
    if ( $$temp >= $$warning ) {
        print
          "$NAME WARNING: +$$temp C | roomtemp=$$temp;$$warning;$$critical\n";
        exit $ERRORS{WARNING};
    }
    print "$NAME OK: +$$temp C | roomtemp=$$temp;$$warning;$$critical\n";
    exit $ERRORS{OK};
}

sub version($$) {
    my ( $progname, $version ) = @_;

    print <<_VERSION;
$progname version $version
Copyright (C) 2011 by Christian Arnold and Schlittermann internet & unix support.

$ME comes with ABSOLUTELY NO WARRANTY. This is free software,
and you are welcome to redistribute it under certain conditions.
See the GNU General Public Licence for details.
_VERSION
}

__END__

=head1 NAME

check_roomtemp - nagios plugin to check room temperature

=head1 SYNOPSIS

check_roomtemp [B<-w>|B<--warning> I<floating_point>] [B<-c>|B<--critical> I<floating_point>]

check_roomtemp [B<-u>|B<--url> I<string>]

check_roomtemp [B<-s>|B<--search> I<string>]

check_roomtemp [B<-h>|B<--help>]

check_roomtemp [B<-m>|B<--man>]

check_roomtemp [B<-V>|B<--version>]

=head1 OPTIONS

=over

=item B<-w>|B<--warning> I<integer>

Exit with WARNING status if temperature is greater than or equal this value (default: 35.00)

=item B<-c>|B<--critical> I<floating point>

Exit with CRITICAL status if temperature is greater than or equal this value (default: 40.00)

=item B<-u>|B<--url> I<string>

URL for geting status information (default: 'http://thermo1.vic.site/status.xml')

=item B<-s>|B<--search> I<string>

Search string for the temperature on the given url (default: '<temp>[+-]\s+([\d\.]+)</temp>')

=item B<-h>|B<--help>

Print detailed help screen.

=item B<-m>|B<--man>

Print manual page.

=item B<-V>|B<--version>

Print version information.

=back

=head1 DESCRIPTION

This nagios plugin check the room temperature.

=head1 VERSION

This man page is current for version 0.1 of B<check_roomtemp>.

=head1 AUTHOR

Written by Christian Arnold L<arnold@schlittermann.de>

=head1 COPYRIGHT

Copyright (C) 2012 by Christian Arnold and Schlittermann internet & unix support.
This is free software, and you are welcome to redistribute it under certain conditions.
See the GNU General Public Licence for details.

=cut
